#!/bin/bash

if [ $# -ne 1 ];then
  echo "Please add the filename of the video as parameter"
  exit 1
fi

SCRIPT_DIRECTORY="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# results are not faster with 4 threads
# PARALLEL_CPU_CORES=4

PARALLEL_CPU_CORES=2

video="$1"

video_directory="$( cd "$( dirname "${video}" )" && pwd )"
file_base_name=$(basename "$video")
file_base_name_without_extenstion=$(basename "$video" | cut -d. -f1)

face_directory="${video_directory}/${file_base_name_without_extenstion}/face_recognition/"

if [ ! -d "$face_directory" ];then
  mkdir -p "$face_directory"
fi

cd "$face_directory"
if [ ! -h "$file_base_name" ];then
  ln -s ../../"$file_base_name" .
fi

function create_images_from_video_with_ffmpeg {
  echo "Creating images of every second of the video. This can take several minutes. Please be patient."

  mkdir images

  if [ "$(ls -A images)" != "" ];then
    echo "Directory for images already filled. Delete it manually if you want to start ffmpeg again."
    return
  fi

  if [ "$(ls -A share/lemke_1)" != "" ];then
    echo "Directory for images share/lemke_1 already filled. Delete it manually if you want to start ffmpeg again."
    return
  fi

  startSeconds=$(date +"%s")
  echo "Start at:"
  date

  # create an image every second
  ffmpeg -i "$file_base_name" -vf fps=1 images/"$file_base_name_without_extenstion"_%04d.png

  endSeconds=$(date +"%s")

  let duration=endSeconds-startSeconds
  let durationMinutes=(duration+59)/60

  echo "End at:"
  date
  echo "ffmpeg finished after: $duration seconds  resp.  $durationMinutes minutes" | tee time_ffmpeg.log

}

function align_faces_from_images {
  echo "Trying to find faces in the images. This can take several minutes. Please be patient."

  # echo "=== $PWD"

  # create directory for selected images and aligned faces
  for index in $(seq 1 $PARALLEL_CPU_CORES);do
    mkdir -p share/lemke_$index
    mkdir -p share/aligned_$index
  done

  if [ "$(ls -A share/aligned_1)" != "" ];then
    echo "Directory for aligned faces already filled. Delete it manually if you want to align again."
    return
  fi

  # directory for the remaining images
  mkdir share/remaining/

  # copy recognized pattern
  cp -a "${SCRIPT_DIRECTORY}/pattern_lemke" share/

  # maybe just use some selected images from the voice part => would be faster

  part=$(expr $(ls -1 images/*png | wc -l) \/ $PARALLEL_CPU_CORES)

  # move images to their directories
  for index in $(seq 1 $PARALLEL_CPU_CORES); do
    let end_image=index*part
    let start_image=end_image-part+1
    echo "$start_image $end_image"

    # move ranges of images
    for image_index in $(seq $start_image $end_image); do

      # use every second image only - faster
      if ((image_index % 2)); then
        filename=$(printf "${file_base_name_without_extenstion}_%04d.png\n" $image_index); mv images/"$filename" share/lemke_$index;
      fi
    done
  done

  # move the rest of the images to the remaining directory
  mv images/* share/remaining/

  startSeconds=$(date +"%s")
  echo "Start at:"
  date

  rm -f "${face_directory}"/share/finito_*txt

  # start aligning of faces
  for index in $(seq 1 $PARALLEL_CPU_CORES); do
    "${SCRIPT_DIRECTORY}/alignFace.exp" "${face_directory}/share" $index &
  done

  while [ $(ls -1 "${face_directory}"/share/finito_*.txt 2>/dev/null | wc -l) -ne $PARALLEL_CPU_CORES ];do
    sleep 1
  done

  endSeconds=$(date +"%s")

  let duration=endSeconds-startSeconds
  let durationMinutes=(duration+59)/60

  echo "End at:"
  date
  echo "aligning faces finished after: $duration seconds  resp.  $durationMinutes minutes" | tee time_align_faces.log
}


function identify_faces {
  echo "Trying to identify faces in the images. This can take several minutes. Please be patient."

  # copy recognized pattern ( duplicate )
  cp -a "${SCRIPT_DIRECTORY}/pattern_lemke" share/

  startSeconds=$(date +"%s")
  echo "Start at:"
  date

  rm -f "${face_directory}"/share/finito_*txt

  # start aligning of faces
  for index in $(seq 1 $PARALLEL_CPU_CORES); do
    "${SCRIPT_DIRECTORY}/identifyFace.exp" "${face_directory}/share" $index &
  done

  while [ $(ls -1 "${face_directory}"/share/finito_*.txt 2>/dev/null | wc -l) -ne $PARALLEL_CPU_CORES ];do
    sleep 1
  done

  endSeconds=$(date +"%s")

  let duration=endSeconds-startSeconds
  let durationMinutes=(duration+59)/60

  echo "End at:"
  date
  echo "identifying faces finished after: $duration seconds  resp.  $durationMinutes minutes" | tee time_identify_faces.log

  # stop all docker containers
  docker stop $(docker ps -a -q)
}

function createCutList {
  grep "lemke.*confide" share/recognized_*txt | sed 's|^.*recognized_\([0-9]*\)\.txt.*\([01]\)\.\([0-9]*\) .*|seconds: \1, confidence: \2\3|g' | sed 's| 0*| |g' | while read line; do
    confidence=$(echo "$line" | sed 's|.*confidence: ||g')

    # threshold for recognition
    if [ "$confidence" -gt 85 ];then
      echo "$line"
    fi
  done > identified.txt

  "${SCRIPT_DIRECTORY}/parseFaceRecognition.rb" "${face_directory}/identified.txt" > parsed_face.txt

  cp ../voice_recognition/*/combined_voices.html combined_recognition.html
  echo "<div style=\"position: absolute; top: 245px; left: 20px\">identified face:</div>" > parsed_face.html
  grep "<div" parsed_face.txt >> parsed_face.html
  cat parsed_face.html >> combined_recognition.html
  grep Final parsed_face.txt > face.txt
}

function combineVoiceAndFaceRecognitionTimes {
  cd "../voice_recognition/${file_base_name_without_extenstion}/"

  rm -f voice_check.txt

  for filename in $(ls -1S *wav);do
    search_string=$(echo $filename | sed 's|S\([0-9]*\)\.wav|\1|g')
    grep ", $search_string$" final.txt > voice_$search_string.txt
    overlappingSeconds=$("${SCRIPT_DIRECTORY}/combineVoiceAndFaceRecognition.rb" ../../face_recognition/face.txt voice_$search_string.txt | tail -n 1)
    echo "$overlappingSeconds $search_string" >> voice_check.txt
  done

  best_matching_voice=$(sort -n voice_check.txt | tail -n 1 | cut -f 2 -d ' ')

  # this is our final cutlist
  "${SCRIPT_DIRECTORY}/combineVoiceAndFaceRecognition.rb" ../../face_recognition/face.txt voice_$best_matching_voice.txt combine > best_matching_voice_combined_cutlist.txt

  # write some HTML
  echo "<div style=\"position: absolute; top: 380px; left: 20px\">combined face and voice recognition:</div>" > combined_voice_and_face.html
  "${SCRIPT_DIRECTORY}/parseTimes.sh" best_matching_voice_combined_cutlist.txt 410 darkgreen >> combined_voice_and_face.html
  cat combined_voices.html "${SCRIPT_DIRECTORY}/scale.html" ../../face_recognition/parsed_face.html combined_voice_and_face.html > ../../face_recognition/combined_recognition.html

  ln -s ../voice_recognition/${file_base_name_without_extenstion}/best_matching_voice_combined_cutlist.txt ../../face_recognition/

  cd ../../face_recognition
}

function convertSeconds {
  ((h=${1}/3600))
  ((m=(${1}%3600)/60))
  ((s=${1}%60))

  printf "%02d:%02d:%02d,000" $h $m $s
}

function createSubtitleList {

  # create subtitles for mplayer
  subtitle_filename="../../${file_base_name_without_extenstion}.srt"
  rm -f "$subtitle_filename"

  index=0
  cat best_matching_voice_combined_cutlist.txt | while read line; do
    let index++
    startSeconds=$(echo $line | cut -f1 -d '-')
    endSeconds=$(echo $line | cut -f2 -d '-')
    echo "$index" >> "$subtitle_filename"
    startTime=$(convertSeconds ${startSeconds})
    endTime=$(convertSeconds ${endSeconds})
    echo "$startTime --> $endTime" >> "$subtitle_filename"
    echo -e "lemke\n" >> "$subtitle_filename"
  done

}

create_images_from_video_with_ffmpeg

align_faces_from_images

identify_faces

createCutList

combineVoiceAndFaceRecognitionTimes

createSubtitleList

