#!/bin/bash

#
# Possible script names:
#
# cutVideo.sh - cut video given in an srt file (subtitles)
# cutVideoHeuteplus.sh - cut video from news show heuteplus / heute+
# downloadAndCutVideo.sh - download the video from heuteplus / heute+ provided in srt file
# playSRT.sh - play video when srt file (subtitles) is given
#

if [ $# -ne 1 ];then
  if [ "$(basename $0)" == "downloadAndCutVideo.sh" ];then
    echo "Please add the filename of the <subtitle.srt> as parameter."
  else
    echo "Please add the filename of the <video> as parameter."
  fi
  exit 1
fi

cutVideoHeuteplus="no"

# download the video if it does not exist
# the URL is in the last line of the srt file

if [ "$(basename $0)" == "downloadAndCutVideo.sh" ];then
  cutVideoHeuteplus="yes"
  video=$(echo "$1" | sed 's|\.[^\.]*$|.mp4|')
  downloadURL=$(tail -n 1 "$1")
  # check if video exists
  if [ ! -e "$video" ];then
    wget "$downloadURL" -O "$video"
    if [ $? -ne 0 ];then
      echo "Download of video \"$downloadURL\" failed."
      exit 2
    fi
  fi
else
  video="$1"
fi

if [ "$(basename $0)" == "cutVideoHeuteplus.sh" ];then
  cutVideoHeuteplus="yes"
fi

echo "Video: $video"

video_start_name=$(echo "$video" | sed 's|\.[^\.]*$||')

srt_filename="${video_start_name}.srt"

# check if file for subtitles exists
if [ ! -e "$srt_filename" ];then
  echo "$srt_filename does not exist"
fi

segment_filename="${video_start_name}_segments.txt"
rm -f "$segment_filename"

function playSubtitlesWithMplayer {
  startPosition=$1
  duration=$2

  # echo "mplayer -idx -fs -ss $startPosition -endpos $duration $video"

  mplayer -idx -fs -ss $startPosition -endpos $duration "$video"
}

function cutVideoHeuteplus {
  startPositionSeconds=$1
  endPositionSeconds=$2

  # for heuteplus we know that there are key frames every 2 seconds
  # therefore use an even number of seconds for the start
  if ((startPositionSeconds % 2)); then
    let startPositionSeconds--
  fi

  echo "file '$video'" >> "$segment_filename"
  echo "inpoint $startPositionSeconds" >> "$segment_filename"
  echo "outpoint $endPositionSeconds" >> "$segment_filename"
}

function cutVideo {
  startPositionSeconds=$1
  duration=$2

  # reencode AAC to mp3 to keep video and audio synchronized
  ffmpeg -y -ss $startPositionSeconds -t $duration -vcodec copy -acodec mp3 -b:a 192k output.mp4 -i "$video"

  # find key frame (this is better for cutting a video)
  startTime=$(ffprobe -show_frames -select_streams v -print_format json=c=1 output.mp4 2>&1 | grep -m 1 "key_frame\": 1" | sed 's|.*pkt_dts_time": "\([^"]*\)".*|\1|g')

  echo -e "\n++++++++++++++ $index $startTime $duration\n\n"

  # cut output again at the key frame
  ffmpeg -y -ss $startTime -t $duration -codec copy output2.mp4 -i output.mp4

  rm -f output.mp4
  mv output2.mp4 ${video}_${formattedIndex}.mp4
}

# filter playlist and read into string
times=$(grep "\-\->" "$srt_filename" | tr '\n' '@')

number_of_playlist_entries=$(grep "\-\->" "$srt_filename" | wc -l)

# for every entry in the list of subtitles / video snippets
for index in $(seq 1 $number_of_playlist_entries); do

  videoSnippet=$(echo -n "$times" | tr '@' '\n' | sed "${index}q;d")
  formattedIndex=$(printf "%03d" $index)

  startPosition=${videoSnippet:0:8}
  endPosition=${videoSnippet:17:8}

  endPositionSeconds=$(echo "$endPosition" | awk -F: '{ print ($1 * 3600) + ($2 * 60) + $3 }')
  startPositionSeconds=$(echo "$startPosition" | awk -F: '{ print ($1 * 3600) + ($2 * 60) + $3 }')
  let duration=endPositionSeconds-startPositionSeconds

  if [ "$(basename $0)" == "playSRT.sh" ];then
    playSubtitlesWithMplayer $startPosition $duration
  elif [ "$(basename $0)" == "cutVideo.sh" ];then
    cutVideo $startPositionSeconds $duration
  elif [ "$cutVideoHeuteplus" == "yes" ];then
    cutVideoHeuteplus $startPositionSeconds $endPositionSeconds
  fi

done

if [ "$cutVideoHeuteplus" == "yes" ];then

  # cut and combine video snippets
  ffmpeg -y -f concat -i "$segment_filename" -c copy "${video_start_name}_combined.mp4"

  # extract audio
  ffmpeg -y -i "${video_start_name}_combined.mp4" -vn -acodec copy "${video_start_name}_audio.aac"

  # convert to mp3 fixed bitrate for convenience
  ffmpeg -y -i "${video_start_name}_audio.aac" -b:a 192k "${video_start_name}_audio.mp3"

elif [ "$(basename $0)" == "cutVideo.sh" ];then

  ls -1 "${video}"_*.mp4 | sed "s|\(.*\)|file '\1'|g" > "${video}_list.txt"

  # concatenate video parts
  ffmpeg -y -f concat -safe 0 -i "${video}_list.txt" -c copy "${video}_combined.mp4"

  # extract audio
  ffmpeg -y -i "${video}_combined.mp4" -vn -acodec copy "${video}_audio.mp3"

  # remove the video snippets
  rm -f "${video}"_[0-9]*.mp4
fi
